@@ -1,34 +1,218 @@
 use core::str::FromStr;
-use std::collections::HashSet;
 
 use said::derivation::SelfAddressing;
-use serde::{Deserialize, Serialize};
+use serde::{Deserialize, Deserializer, Serialize};
 use std::collections::HashMap;
 use wasm_bindgen::prelude::*;
 
 mod capture_base;
 mod overlay;
+pub mod validator;
 use crate::state::capture_base::CaptureBase;
 use crate::state::overlay::Overlay;
 
+type DynOverlay = Box<dyn Overlay>;
+
+impl<'de> Deserialize<'de> for DynOverlay {
+    fn deserialize<D>(deserializer: D) -> Result<Self, D::Error>
+    where
+        D: Deserializer<'de>,
+    {
+        let de_overlay = serde_value::Value::deserialize(deserializer)?;
+        if let serde_value::Value::Map(ref overlay) = de_overlay {
+            if let Some(serde_value::Value::String(overlay_type)) =
+                overlay.get(&serde_value::Value::String("type".to_string()))
+            {
+                if overlay_type.contains("/character_encoding/") {
+                    return Ok(Box::new(
+                        de_overlay
+                            .deserialize_into::<overlay::CharacterEncoding>()
+                            .unwrap(),
+                    ));
+                } else if overlay_type.contains("/entry/") {
+                    return Ok(Box::new(
+                        de_overlay.deserialize_into::<overlay::Entry>().unwrap(),
+                    ));
+                } else if overlay_type.contains("/entry_code/") {
+                    return Ok(Box::new(
+                        de_overlay.deserialize_into::<overlay::EntryCode>().unwrap(),
+                    ));
+                } else if overlay_type.contains("/format/") {
+                    return Ok(Box::new(
+                        de_overlay.deserialize_into::<overlay::Format>().unwrap(),
+                    ));
+                } else if overlay_type.contains("/information/") {
+                    return Ok(Box::new(
+                        de_overlay
+                            .deserialize_into::<overlay::Information>()
+                            .unwrap(),
+                    ));
+                } else if overlay_type.contains("/label/") {
+                    return Ok(Box::new(
+                        de_overlay.deserialize_into::<overlay::Label>().unwrap(),
+                    ));
+                } else if overlay_type.contains("/unit/") {
+                    return Ok(Box::new(
+                        de_overlay.deserialize_into::<overlay::Unit>().unwrap(),
+                    ));
+                } else if overlay_type.contains("/meta/") {
+                    return Ok(Box::new(
+                        de_overlay.deserialize_into::<overlay::Meta>().unwrap(),
+                    ));
+                } else {
+                    return Err(serde::de::Error::custom(format!(
+                        "unknown overlay type: {}",
+                        overlay_type
+                    )));
+                }
+            } else {
+                return Err(serde::de::Error::missing_field("type"));
+            }
+        }
+
+        Err(serde::de::Error::custom(format!(
+            "overlay must be an object, got: {:?}",
+            de_overlay
+        )))
+    }
+}
+
 #[derive(Serialize)]
 pub struct OCA {
     pub capture_base: CaptureBase,
-    pub overlays: Vec<Box<dyn Overlay>>,
+    pub overlays: Vec<DynOverlay>,
     #[serde(skip)]
-    translations: HashMap<Language, OCATranslation>,
-    #[serde(skip)]
-    errors: Vec<Error>,
+    meta_translations: HashMap<Language, OCATranslation>,
 }
 
-#[derive(Debug)]
-pub struct Error {
-    pub msg: String,
-}
+impl<'de> Deserialize<'de> for OCA {
+    fn deserialize<D>(deserializer: D) -> Result<Self, D::Error>
+    where
+        D: Deserializer<'de>,
+    {
+        let de_oca = serde_value::Value::deserialize(deserializer)?;
+        if let serde_value::Value::Map(ref oca) = de_oca {
+            let capture_base;
+            let overlays;
+            let mut meta_translations: HashMap<Language, OCATranslation> = HashMap::new();
 
-impl Error {
-    fn new(msg: String) -> Error {
-        Error { msg }
+            match oca.get(&serde_value::Value::String("capture_base".to_string())) {
+                Some(de_capture_base) => {
+                    capture_base = de_capture_base
+                        .clone()
+                        .deserialize_into::<CaptureBase>()
+                        .unwrap();
+                }
+                None => return Err(serde::de::Error::missing_field("capture_base")),
+            }
+            match oca.get(&serde_value::Value::String("overlays".to_string())) {
+                Some(de_overlays) => {
+                    if let serde_value::Value::Seq(de_overlays_value) = de_overlays {
+                        let meta_overlay_positions: Vec<bool> = de_overlays_value
+                            .iter()
+                            .map(|x| {
+                                if let serde_value::Value::Map(de_overlay_value) = x {
+                                    if let Some(serde_value::Value::String(overlay_type)) =
+                                        de_overlay_value
+                                            .get(&serde_value::Value::String("type".to_string()))
+                                    {
+                                        overlay_type.contains("/meta/")
+                                    } else {
+                                        false
+                                    }
+                                } else {
+                                    false
+                                }
+                            })
+                            .collect();
+                        let mut meta_overlays_iter = meta_overlay_positions.iter();
+                        let mut meta_overlays: Vec<_> = de_overlays_value.clone();
+                        meta_overlays.retain(|_| *meta_overlays_iter.next().unwrap());
+                        let mut rest_overlays_iter = meta_overlay_positions.iter();
+                        let mut rest_overlays: Vec<_> = de_overlays_value.clone();
+                        rest_overlays.retain(|_| !*rest_overlays_iter.next().unwrap());
+
+                        for meta_overlay in meta_overlays {
+                            if let serde_value::Value::Map(meta_overlay_value) = meta_overlay {
+                                let language;
+                                let name;
+                                let description;
+                                match meta_overlay_value
+                                    .get(&serde_value::Value::String("language".to_string()))
+                                {
+                                    Some(de_language) => {
+                                        language = de_language
+                                            .clone()
+                                            .deserialize_into::<Language>()
+                                            .unwrap();
+                                    }
+                                    None => {
+                                        return Err(serde::de::Error::missing_field(
+                                            "language in meta overlay",
+                                        ))
+                                    }
+                                }
+                                match meta_overlay_value
+                                    .get(&serde_value::Value::String("name".to_string()))
+                                {
+                                    Some(de_name) => {
+                                        name =
+                                            de_name.clone().deserialize_into::<String>().unwrap();
+                                    }
+                                    None => {
+                                        return Err(serde::de::Error::missing_field(
+                                            "name in meta overlay",
+                                        ))
+                                    }
+                                }
+                                match meta_overlay_value
+                                    .get(&serde_value::Value::String("description".to_string()))
+                                {
+                                    Some(de_description) => {
+                                        description = de_description
+                                            .clone()
+                                            .deserialize_into::<String>()
+                                            .unwrap();
+                                    }
+                                    None => {
+                                        return Err(serde::de::Error::missing_field(
+                                            "language in meta overlay",
+                                        ))
+                                    }
+                                }
+                                let mut t = OCATranslation::new();
+                                if !name.trim().is_empty() {
+                                    t.add_name(name);
+                                }
+                                if !description.trim().is_empty() {
+                                    t.add_description(description);
+                                }
+                                meta_translations.insert(language, t);
+                            }
+                        }
+
+                        let de_rest_overlays = serde_value::Value::Seq(rest_overlays);
+                        overlays = de_rest_overlays
+                            .deserialize_into::<Vec<DynOverlay>>()
+                            .unwrap();
+                    } else {
+                        return Err(serde::de::Error::custom("overlays must be an array"));
+                    }
+                }
+                None => return Err(serde::de::Error::missing_field("overlay")),
+            }
+
+            Ok(OCA {
+                capture_base,
+                overlays,
+                meta_translations,
+            })
+        } else {
+            Err(serde::de::Error::custom(format!(
+                "OCA must be an object, got: {:?}",
+                de_oca
+            )))
+        }
     }
 }
 
@@ -37,44 +221,43 @@ impl OCA {
         OCA {
             capture_base: CaptureBase::new(),
             overlays: vec![overlay::CharacterEncoding::new(&default_encoding)],
-            translations: HashMap::new(),
-            errors: vec![],
+            meta_translations: HashMap::new(),
         }
     }
 
-    pub fn enforce_translations(mut self, languages: Vec<Language>) -> OCA {
-        for lang in languages.iter() {
-            self.translations.insert(*lang, OCATranslation::new());
-        }
-        self
-    }
-
     pub fn add_name(mut self, names: HashMap<Language, String>) -> OCA {
-        self.validate_translations(&names);
-
         for (lang, name) in names.iter() {
-            if let Some(t) = self.translations.get_mut(lang) {
-                t.add_name(name.clone());
+            match self.meta_translations.get_mut(lang) {
+                Some(t) => {
+                    t.add_name(name.clone());
+                }
+                None => {
+                    let mut t = OCATranslation::new();
+                    t.add_name(name.clone());
+                    self.meta_translations.insert(*lang, t);
+                }
             }
         }
         self
     }
 
     pub fn add_description(mut self, descriptions: HashMap<Language, String>) -> OCA {
-        self.validate_translations(&descriptions);
-
         for (lang, description) in descriptions.iter() {
-            if let Some(t) = self.translations.get_mut(lang) {
-                t.add_description(description.clone());
+            match self.meta_translations.get_mut(lang) {
+                Some(t) => {
+                    t.add_description(description.clone());
+                }
+                None => {
+                    let mut t = OCATranslation::new();
+                    t.add_description(description.clone());
+                    self.meta_translations.insert(*lang, t);
+                }
             }
         }
         self
     }
 
     pub fn add_attribute(mut self, attr: Attribute) -> OCA {
-        self.validate_translations(&attr.translations);
-        self.validate_attribute(&attr);
-
         self.capture_base.add(&attr);
 
         if attr.encoding.is_some() {
@@ -182,12 +365,8 @@ impl OCA {
         self
     }
 
-    pub fn finalize(mut self) -> Result<OCA, Vec<Error>> {
-        if !self.errors.is_empty() {
-            return Err(self.errors);
-        }
-
-        for (lang, translation) in self.translations.iter() {
+    pub fn finalize(mut self) -> OCA {
+        for (lang, translation) in self.meta_translations.iter() {
             self.overlays.push(overlay::Meta::new(lang, translation));
         }
 
@@ -196,150 +375,7 @@ impl OCA {
         for o in self.overlays.iter_mut() {
             o.sign(&sai);
         }
-        Ok(self)
-    }
-}
-
-impl OCA {
-    fn validate_translations<T>(&mut self, translations: &HashMap<Language, T>) {
-        if self.translations.is_empty() {
-            self.errors.push(Error::new(
-                "enforced translations are not defined".to_string(),
-            ));
-            return;
-        }
-
-        let translation_langs: HashSet<Language> = translations.keys().cloned().collect();
-        let enforced_langs: HashSet<Language> = self.translations.keys().cloned().collect();
-        if !translation_langs.eq(&enforced_langs) {
-            let missing_enforcement: HashSet<&Language> =
-                translation_langs.difference(&enforced_langs).collect();
-            for m in missing_enforcement {
-                self.errors.push(Error::new(
-                    format!("translations for {:?} language are not enforced", m).to_string(),
-                ));
-            }
-
-            let missing_translations: HashSet<&Language> =
-                enforced_langs.difference(&translation_langs).collect();
-            for m in missing_translations {
-                self.errors.push(Error::new(
-                    format!("translations for {:?} language are missing", m).to_string(),
-                ));
-            }
-        }
-    }
-
-    fn validate_attribute(&mut self, attribute: &Attribute) {
-        if self.translations.is_empty() && !attribute.translations.is_empty() {
-            self.errors.push(Error::new(
-                "enforced translations are not defined".to_string(),
-            ));
-            return;
-        }
-
-        let enforced_langs: HashSet<Language> = self.translations.keys().cloned().collect();
-        let label_langs: HashSet<Language> = attribute
-            .translations
-            .iter()
-            .filter(|(_, t)| t.label.is_some())
-            .map(|(l, _)| *l)
-            .collect();
-        if !label_langs.is_empty() {
-            let missing_enforcement: HashSet<&Language> =
-                label_langs.difference(&enforced_langs).collect();
-            for m in missing_enforcement {
-                self.errors.push(Error::new(
-                    format!(
-                        "in '{}' attribute: label translations for {:?} language are not enforced",
-                        attribute.name, m
-                    )
-                    .to_string(),
-                ));
-            }
-
-            let missing_translations: HashSet<&Language> =
-                enforced_langs.difference(&label_langs).collect();
-            for m in missing_translations {
-                self.errors.push(Error::new(
-                    format!(
-                        "in '{}' attribute: label translations for {:?} language are missing",
-                        attribute.name, m
-                    )
-                    .to_string(),
-                ));
-            }
-        }
-        let entries_langs: HashSet<Language> = attribute
-            .translations
-            .iter()
-            .filter(|(_, t)| t.entries.is_some())
-            .map(|(l, _)| *l)
-            .collect();
-        if !entries_langs.is_empty() {
-            if let Some(entry_codes) = &attribute.entry_codes {
-                let entry_ids: HashSet<String> = entry_codes.iter().cloned().collect();
-                for (l, t) in attribute.translations.iter() {
-                    if let Some(e) = &t.entries {
-                        let lang_entry_ids: HashSet<String> = e.keys().cloned().collect();
-                        let missing_entry_tr: HashSet<&String> =
-                            entry_ids.symmetric_difference(&lang_entry_ids).collect();
-                        for m in missing_entry_tr {
-                            self.errors.push(
-                                Error::new(format!("in '{}' attribute: '{}' entry translations for {:?} language are missing", attribute.name, m, l).to_string())
-                            );
-                        }
-                    }
-                }
-            }
-
-            let missing_enforcement: HashSet<&Language> =
-                entries_langs.difference(&enforced_langs).collect();
-            for m in missing_enforcement {
-                self.errors.push(
-                    Error::new(format!("in '{}' attribute: entries translations for {:?} language are not enforced", attribute.name, m).to_string())
-                );
-            }
-
-            let missing_translations: HashSet<&Language> =
-                enforced_langs.difference(&entries_langs).collect();
-            for m in missing_translations {
-                self.errors.push(Error::new(
-                    format!(
-                        "in '{}' attribute: entries translations for {:?} language are missing",
-                        attribute.name, m
-                    )
-                    .to_string(),
-                ));
-            }
-        }
-        let info_langs: HashSet<Language> = attribute
-            .translations
-            .iter()
-            .filter(|(_, t)| t.information.is_some())
-            .map(|(l, _)| *l)
-            .collect();
-        if !info_langs.is_empty() {
-            let missing_enforcement: HashSet<&Language> =
-                info_langs.difference(&enforced_langs).collect();
-            for m in missing_enforcement {
-                self.errors.push(
-                    Error::new(format!("in '{}' attribute: information translations for {:?} language are not enforced", attribute.name, m).to_string())
-                );
-            }
-
-            let missing_translations: HashSet<&Language> =
-                enforced_langs.difference(&info_langs).collect();
-            for m in missing_translations {
-                self.errors.push(Error::new(
-                    format!(
-                        "in '{}' attribute: information translations for {:?} language are missing",
-                        attribute.name, m
-                    )
-                    .to_string(),
-                ));
-            }
-        }
+        self
     }
 }
 
@@ -458,7 +494,7 @@ impl Entry {
     }
 }
 
-#[derive(Serialize, Deserialize, Debug, Clone)]
+#[derive(Serialize, Deserialize, Debug, Clone, Hash, Eq, PartialEq)]
 pub struct OCATranslation {
     name: Option<String>,
     description: Option<String>,
@@ -589,8 +625,7 @@ mod tests {
 
     #[test]
     fn test_2() {
-        let mut oca = OCA::new(Encoding::Utf8)
-            .enforce_translations(vec![Language::En, Language::Pl])
+        let oca = OCA::new(Encoding::Utf8)
             .add_name(hashmap! {
                 Language::En => "Driving Licence".to_string(),
                 Language::Pl => "Prawo Jazdy".to_string(),
@@ -598,16 +633,10 @@ mod tests {
             .add_description(hashmap! {
                 Language::En => "Driving Licence".to_string(),
                 Language::Pl => "Prawo Jazdy".to_string(),
-            });
+            })
+            .finalize();
 
-        oca = oca.finalize().unwrap_or_else(|errors| {
-            for err in errors.iter() {
-                eprintln!("Error: {:?}", err);
-            }
-            std::process::exit(1)
-        });
-
-        println!("{:#?}", serde_json::to_string(&oca).unwrap());
+        // println!("{:#?}", serde_json::to_string(&oca).unwrap());
 
         assert_eq!(oca.capture_base.attributes.len(), 0);
     }
@@ -615,7 +644,6 @@ mod tests {
     #[test]
     fn test_1() {
         let mut oca = OCA::new(Encoding::Utf8)
-            .enforce_translations(vec![Language::En, Language::Pl])
             .add_name(hashmap! {
                 Language::En => "Driving Licence".to_string(),
                 Language::Pl => "Prawo Jazdy".to_string(),
@@ -661,18 +689,9 @@ mod tests {
             .add_encoding(Encoding::Iso8859_1)
             .add_format("DD/MM/YYYY".to_string());
 
-        oca = oca
-            .add_attribute(attr1)
-            .add_attribute(attr2)
-            .finalize()
-            .unwrap_or_else(|errors| {
-                for err in errors.iter() {
-                    eprintln!("Error: {:?}", err.msg);
-                }
-                std::process::exit(1)
-            });
+        oca = oca.add_attribute(attr1).add_attribute(attr2).finalize();
 
-        println!("{:#?}", serde_json::to_string(&oca).unwrap());
+        // println!("{:#?}", serde_json::to_string(&oca).unwrap());
 
         assert_eq!(oca.capture_base.attributes.len(), 2);
         assert_eq!(oca.capture_base.pii.len(), 1);
