@@ -22,6 +22,9 @@ impl Overlay for InformationOverlay {
     fn language(&self) -> Option<&Language> {
         Some(&self.language)
     }
+    fn attributes(&self) -> Vec<&String> {
+        self.attr_information.keys().collect::<Vec<&String>>()
+    }
 
     fn add(&mut self, attribute: &Attribute) {
         if let Some(tr) = attribute.translations.get(&self.language) {
